import json

from . import macfinder
from net_suite.model import *
from net_suite import app
from net_suite.views import api_netdb_required, get_db_conn, db, get_current_mgr
import requests
from flask import request, jsonify, abort
import re
from . import OUI_DB
import redis


class MacFinderJob(DBObject):
    def __init__(self, job_id, mac, vlan, bcd, started=None, finished=False, failed=False, status=None, vendor=None):
        self.job_id = job_id
        self.finished = finished
        self.failed= failed
        self.status = status
        self.mac = format_mac(mac)
        self.vendor = OUI_DB.get(self.mac[:8], None) if vendor is None else vendor
        self.vlan = vlan.id if type(vlan) == VLAN else vlan
        self.bcd = bcd
        self.started = started if started is not None else datetime.datetime.now()

    def poll(self, session, db, connection):
        if self.finished:
            return self.status
        self.status = session.get(app.config.get('MACFINDER_SERVER_URL') + '/status/{}'.format(self.job_id),
                                  cert=(app.config.get('MACFINDER_CERT'), app.config.get('MACFINDER_KEY')),
                                  verify=app.config.get('MACFINDER_CA')).json()
        self.finished = self.status['status'] == 'finished'
        self.failed = self.status['status'] == 'failed'
        return self.status


@macfinder.route('/job', methods=['post'])
@api_netdb_required
def create_job():
    if not len(format_mac(request.json['mac'])) == 17:
        return jsonify({'success': False, 'error': 'invalid_mac'}), 400
    vlan_valid = False
    vlans = VLAN.get_by_bcd(db, get_db_conn(), request.json['bcd'])
    user = get_current_mgr()
    for vlan in vlans:
        if not user.has_permission('nm.macfinder_global'):
            if user.has_bcd(vlan.bcd):
                vlan_valid = True
                break
        else:
            vlan_valid = True
    if not vlan_valid:
        return jsonify({'success': False, 'error': 'invalid_vlan'}), 404
    created_jobs = []
    for vlan in vlans:
        with redis.Redis(db=int(app.config.get('REDIS_DB_INDEX'))) as r:
            resp = requests.post(app.config.get('MACFINDER_SERVER_URL') + '/new_task',
                                 cert=(app.config.get('MACFINDER_CERT'), app.config.get('MACFINDER_KEY')),
                                 verify=app.config.get('MACFINDER_CA'),
                                 data={'mac': request.json['mac'], 'vlan': vlan.id, 'net_instnc': vlan.net_instnc,
                                       'tag_check': 'y'})
            if not resp.ok:
                return jsonify({'success': False, 'error': 'internal_error'}), 500
            job_id = resp.json()['job_id']
            created_jobs.append(job_id)
            pipe = r.pipeline()
            pipe.multi()
            pipe.set(job_id, app.json_encoder().encode(MacFinderJob(job_id=job_id, mac=request.json['mac'], vlan=vlan, bcd=request.json['bcd'])))
            pipe.expire(job_id, 60 * 60 * 12)
            pipe.execute()
    return jsonify({'success': True, 'job_ids': created_jobs})


@macfinder.route('/job/<job_id>')
@api_netdb_required
def poll(job_id):
    with redis.Redis(db=int(app.config.get('REDIS_DB_INDEX'))) as r:
        job = MacFinderJob(**json.loads(r.get(job_id)))
        session = requests.session()
        job.poll(session, db, get_db_conn())
        pipe = r.pipeline()
        pipe.multi()
        pipe.set(job_id, app.json_encoder().encode(job))
        pipe.expire(job_id, 60 * 60 * 12)
        pipe.execute()
        return jsonify(job)


@macfinder.route('/oui_lookup')
def oui_lookup():
    mac = format_mac(request.args['mac'])
    vendor = OUI_DB.get(mac[:8], None)
    if vendor is None:
        return jsonify({'mac': mac, 'vendor': None}), 404
    return jsonify({'mac': mac, 'vendor': vendor})


def format_mac(mac):
    mac = mac.lower().strip()
    mac = re.sub(r'(.*?)([0-9a-f]+|$|^)', lambda m: ('' * len(m.groups()[0]) + m.groups()[1]), mac)
    return ''.join(':' + c if i % 2 == 0 and not i == 0 else c for i, c in enumerate(mac, 0))
